# @Author: BingWu Yang <detailyang>
# @Date:   2016-03-29T17:47:44+08:00
# @Email:  detailyang@gmail.com
# @Last modified by:   detailyang
# @Last modified time: 2016-03-30T20:53:10+08:00
# @License: The MIT License (MIT)


import ply.yacc as yacc
import eslast as ast

from esllexer import ESLLexer


tokens = ESLLexer.tokens

# expression: URL
#           | URL METHOD
#           | URL METHOD OPTIONS
# OPTIONS:    empty
#           | OPTIONS OPTION
# OPTION:     HEADER
#           | QUERYSTRING
#           | BODY

def p_request(p):
    '''request : URL
           | URL METHOD
           | URL METHOD OPTIONS'''
    if len(p) == 2:
        p[0] = ast.RequestNode(ast.MethodNode('GET'), ast.URLNode(p[1]), None)
    elif len(p) == 3:
        p[0] = ast.RequestNode(ast.MethodNode(p[2]), ast.URLNode(p[1]), None)
    else:
        p[0] = ast.RequestNode(ast.MethodNode(p[2]), ast.URLNode(p[1]), p[3])

def p_options(p):
    '''OPTIONS :
              | OPTION
              | OPTIONS OPTION'''
    if len(p)  == 2:
        p[0] = ast.OptionListNode([p[1]])
    else:
        p[0] = p[1].append(p[2])

def p_option_empty(p):
    ' OPTION :   empty '
    p[0] = p[1]

def p_option_header(p):
    ' OPTION :   HEADER '
    p[0] = ast.HeaderNode(p[1])

def p_option_querystring(p):
    ' OPTION :  QUERYSTRING '
    p[0] = ast.QueryStringNode(p[1])

def p_option_body(p):
    ' OPTION : BODY '
    p[0] = ast.BodyNode(p[1])

def p_empty(p):
    'empty :'
    p[0] = []

def p_error(p):
    print "Syntax error in input!"

def parse(text):
    parser = yacc.yacc()
    ast = parser.parse(text, ESLLexer().build())
    return ast



# @Author: BingWu Yang <detailyang>
# @Date:   2016-03-29T17:46:26+08:00
# @Email:  detailyang@gmail.com
# @Last modified by:   detailyang
# @Last modified time: 2016-03-30T20:51:25+08:00
# @License: The MIT License (MIT)


from ply.lex import TOKEN
import re
import ply.lex as lex


digit            = r'([0-9])'
nondigit         = r'([_A-Za-z])'
variable         = r'[a-zA-Z0-9\[\]-_]+'
header           = r'--h' + variable + r'=' + variable
querystring = r'--q' + variable + r'=' + variable
body = r'--b' + variable + r'=' + variable


class ESLLexer:
    # List of token names.   This is always required
    tokens = (
       'URL',
       'METHOD',
       'HEADER',
       'QUERYSTRING',
       'BODY'
    )

    # Regular expression rules for simple tokens
    # t_HEADER = r'--h' + variable + r'=' + variable
    # t_QUERYSTRING = r'--q' + variable + r'=' + variable
    # t_BODY = r'--b' + variable + r'=' + variable
    def t_URL(self, t):
        r'^[^ ]+'
        return t

    def t_METHOD(self, t):
        r'(GET|get|POST|post|DELETE|delete|OPTIONS|options|CONNECT|connect)'
        t.value = t.value
        return t

    @TOKEN(header)
    def t_HEADER(self, t):
        return t

    @TOKEN(querystring)
    def t_QUERYSTRING(self, t):
        return t

    @TOKEN(body)
    def t_BODY(self, t):
        return t

    # Define a rule so we can track line numbers
    def t_newline(self,t):
        r'\n+'
        t.lexer.lineno += len(t.value)

    # A string containing ignored characters (spaces and tabs)
    t_ignore  = ' \t'

    # Error handling rule
    def t_error(self,t):
        print "Illegal character '%s'" % t.value[0]
        t.lexer.skip(1)

    # Build the lexer
    def build(self,**kwargs):
        self.lexer = lex.lex(module=self, **kwargs)

    # Test it output
    def test(self,data):
        self.lexer.input(data)
        while True:
             tok = self.lexer.token()
             if not tok: break
             print tok



# @Author: BingWu Yang <detailyang>
# @Date:   2016-03-30T20:49:47+08:00
# @Email:  detailyang@gmail.com
# @Last modified by:   detailyang
# @Last modified time: 2016-03-30T20:56:55+08:00
# @License: The MIT License (MIT)


from esl.eslyacc import parse
from eslast import *

__version__ = '0.1.0'




# parsetab.py
# This file is automatically generated. Do not edit.
_tabversion = '3.8'

_lr_method = 'LALR'

_lr_signature = '26C673E7BBE145DDDB6A7C8BE02FCE0C'
    
_lr_action_items = {'BODY':([3,4,5,6,7,8,9,10,],[4,-10,-5,-9,4,-8,-7,-6,]),'URL':([0,],[1,]),'QUERYSTRING':([3,4,5,6,7,8,9,10,],[6,-10,-5,-9,6,-8,-7,-6,]),'HEADER':([3,4,5,6,7,8,9,10,],[8,-10,-5,-9,8,-8,-7,-6,]),'METHOD':([1,],[3,]),'$end':([1,2,3,4,5,6,7,8,9,10,],[-1,0,-2,-10,-5,-9,-3,-8,-7,-6,]),}

_lr_action = {}
for _k, _v in _lr_action_items.items():
   for _x,_y in zip(_v[0],_v[1]):
      if not _x in _lr_action:  _lr_action[_x] = {}
      _lr_action[_x][_k] = _y
del _lr_action_items

_lr_goto_items = {'request':([0,],[2,]),'OPTIONS':([3,],[7,]),'empty':([3,7,],[9,9,]),'OPTION':([3,7,],[5,10,]),}

_lr_goto = {}
for _k, _v in _lr_goto_items.items():
   for _x, _y in zip(_v[0], _v[1]):
       if not _x in _lr_goto: _lr_goto[_x] = {}
       _lr_goto[_x][_k] = _y
del _lr_goto_items
_lr_productions = [
  ("S' -> request","S'",1,None,None,None),
  ('request -> URL','request',1,'p_request','eslyacc.py',27),
  ('request -> URL METHOD','request',2,'p_request','eslyacc.py',28),
  ('request -> URL METHOD OPTIONS','request',3,'p_request','eslyacc.py',29),
  ('OPTIONS -> <empty>','OPTIONS',0,'p_options','eslyacc.py',39),
  ('OPTIONS -> OPTION','OPTIONS',1,'p_options','eslyacc.py',40),
  ('OPTIONS -> OPTIONS OPTION','OPTIONS',2,'p_options','eslyacc.py',41),
  ('OPTION -> empty','OPTION',1,'p_option_empty','eslyacc.py',49),
  ('OPTION -> HEADER','OPTION',1,'p_option_header','eslyacc.py',53),
  ('OPTION -> QUERYSTRING','OPTION',1,'p_option_querystring','eslyacc.py',57),
  ('OPTION -> BODY','OPTION',1,'p_option_body','eslyacc.py',61),
  ('empty -> <empty>','empty',0,'p_empty','eslyacc.py',65),
]
